#!/bin/sh
#***********************************************************************#
#                                                                       #
#                               NOTICE                                  #
#                                                                       #
#                  COPYRIGHT 2012 QLOGIC CORPORATION                    #
#                        ALL RIGHTS RESERVED                            #
#                                                                       #
#  This computer program is CONFIDENTIAL  and contains TRADE SECRETS of #
#  QLOGIC CORPORATION.  The  receipt or possession of this program does #
#  not convey any rights to reproduce or disclose  its contents,  or to #
#  manufacture, use, or sell anything that it may describe, in whole or #
#  in part, without the specific written consent of QLOGIC CORPORATION. #
#  Any reproduction of this program without the express written consent #
#  of QLOGIC CORPORATION  is a violation of the copyright laws and  may #
#  subject you to civil liability and criminal prosecution.             #
#                                                                       #
#***********************************************************************#

#
# ACTION FILE: located in /lib/udev/
#

err() {
                echo "$@" >&2
                if [ -x /bin/logger ]; then
                        /bin/logger -t "${0##*/}[$$]" "$@"
                fi
		if [ -x /usr/bin/logger ]; then
                        /usr/bin/logger -t "${0##*/}[$$]" "$@"
                fi
}

count=0
MAX_COUNT=20
DRV_NAME=qlcnic
SYSFS=/sys
ETH=${FW_DUMP}
QFWD=${SYSFS}/class/net/${ETH}/device/fw_dump
DFILE_PATH=/opt/QLogic_Corporation/FW_Dumps
DFILE=${DFILE_PATH}/${DRV_NAME}_fw_dump_${ETH}_`eval date +%Y%m%d_%H%M%S`.bin

mkdir -p $DFILE_PATH &> /dev/null
# Verify fw_dump binary-attribute file

if ! test -f ${QFWD} ; then
        err "qlcnic: no firmware dump file at interface $ETH!!!"
	err "qlcnic: trying firmware dump extraction using ethtool"
	while [ $count -le $MAX_COUNT ]
	do
		ls /sbin/ethtool &> /dev/null
		if [ $? -eq 0 ]; then
			/sbin/ethtool -i $ETH &> /dev/null
			if [ $? -eq 0 ]; then
				/sbin/ethtool -h | grep "get-dump" | grep "Get dump flag, data" &> /dev/null
				if [ $? -ne 0 ];then
					err "Ethtool command option is not present to extract minidump"
                          		err "Please update your ethtool program in \"/sbin\" directory to extract minidump"
					err "qlcnic: minidump extraction failed by udev script"
                                	exit 1
				else
					rm -f $DFILE
					/sbin/ethtool -w "${ETH}" data $DFILE &> /dev/null
					gzip ${DFILE}
					err "qlcnic: firmware dump saved to file ${DFILE}.gz."
					exit 0
				fi
			else
				(( count++ ))
				sleep 1
			fi
		else
			err "Ethtool program is not present in standard path \"/sbin\""
			err "qlcnic: minidump extraction failed by udev script"
			exit 1
		fi
	done
	err "qlcnic: $ETH interface is not available to extract minidump"
	err "qlcnic: minidump extraction failed by udev script"
	exit 1
fi

# Go with dump
rm -f $DFILE
size=`cat /sys/class/net/${ETH}/device/fwdump_size`
(( remainder = size%4096 ))
(( loops = size/4096 ))
if [ $remainder != "0" ]; then
(( loops++ ))
fi
for (( i = 0; i < $loops ; i++ ))
do
    echo "Page: $i"
    cat /sys/class/net/${ETH}/device/fw_dump >> $DFILE
done

if ! test -s "${DFILE}" ; then
                err "qlcnic: no firmware dump file at interface ${ETH}!!!"
                rm ${DFILE}
                exit 1
fi

gzip ${DFILE}
err "qlcnic: firmware dump saved to file ${DFILE}.gz."
exit 0
