#!/bin/sh
#***********************************************************************#
#                                                                       #
#                               NOTICE                                  #
#                                                                       #
#                  COPYRIGHT 2012 QLOGIC CORPORATION                    #
#                        ALL RIGHTS RESERVED                            #
#                                                                       #
#  This computer program is CONFIDENTIAL  and contains TRADE SECRETS of #
#  QLOGIC CORPORATION.  The  receipt or possession of this program does #
#  not convey any rights to reproduce or disclose  its contents,  or to #
#  manufacture, use, or sell anything that it may describe, in whole or #
#  in part, without the specific written consent of QLOGIC CORPORATION. #
#  Any reproduction of this program without the express written consent #
#  of QLOGIC CORPORATION  is a violation of the copyright laws and  may #
#  subject you to civil liability and criminal prosecution.             #
#                                                                       #
#***********************************************************************#

MODULE=qlcnic.ko
KERN_VER=`uname -r`
LIB_DIR=/lib/modules/${KERN_VER}
BUILD_DIR=${LIB_DIR}/build
INSTALL_DIR=$LIB_DIR/extra
INSTALL_MOD_DIR=updates/
UDEV_OP="/sbin/udevadm control --reload-rules"
UDEV_RULE_DIR=/etc/udev/rules.d
UDEV_RULE_FILE=99-qlcnic.rules
UDEV_SCRIPT_DIR=/lib/udev
DUMP_SCRIPT_DIR=/opt/qlcnic
UDEV_SRC_DIR=extras
UDEV_SCRIPT=qlcnic_udev.sh
DUMP_SCRIPT=ql_dump.sh
UDEV_EARLY_RULE=/etc/udev/rules.d/05-udev-early.rules
UDEV_TMP_RULE=/tmp/tmp.rules
CLIAGENT=netqlremote
QLCNIC_MAN_PAGE=qlcnic.4.gz
QLCNIC_MAN_DIR=/usr/share/man/man4
UDEVINSTALL=1
INPUT_ARGS=1

# Determine platform, driver version and distro
PROC=`uname -p`
KERN=`uname -r | awk -F "$PROC" {'print $1'}`
DISTRO="unknown"

get_os()
{
	echo "** Find distro **"
	os=`cat /etc/issue | grep '(' | awk -F ')' {'print $1'}`
	if [[ $os =~ .*Red*. ]]; then 
		version=`cat /etc/redhat-release | grep 'Red Hat' | awk -F 'release' {'print $2'} | awk -F ' ' {'print $1'} | sed -e "s/.0//g"`
		DISTRO=rhel$version
	elif [[ $os =~ .*SUSE*. ]]; then 
		version=`cat /etc/SuSE-release | grep 'VERSION' | awk -F '=' {'print $2'}`
		patch=`cat /etc/SuSE-release | grep 'PATCHLEVEL' | awk -F '=' {'print $2'}`
		if [ $patch -eq "0" ]; then
			DISTRO=sles${version/ /}
		else
			DISTRO=sles${version/ /}sp${patch/ /}
		fi
		INSTALL_DIR=$LIB_DIR/updates
	else
		os=`cat /etc/issue | grep -e 'XenServer' -e 'Oracle Linux Server release'`
		if [ $? -ne 0 ];then
			echo "** Unsupported $DISTRO distro! Aborting install **"
			exit
		fi
	fi
	echo " ** Find udev adm type **"
	udevd=`ls -St /sbin/udev* | grep 'udevcontrol'`
	if [ -n "$udevd" ]; then
		UDEV_OP="udevcontrol reload_rules"
	else
		udevd=`ls -St /sbin/udev* | grep 'udevadm'`
		if [ -z "$udevd" ]; then
			UDEVINSTALL=0
		fi
	fi
}


build_driver()
{
	get_os
	#Build driver
	echo "** Building driver from source files **"
	make -C ${BUILD_DIR} M=$PWD modules CONFIG_QLCNIC=m
}

install_driver()
{
	echo "** Installing the $MODULE modules to ${INSTALL_DIR} **"
	if [[ $os =~ .*Red*. ]]; then
		make -C /lib/modules/`uname -r`/build M=${PWD} modules_install
	elif [[ $os =~ .*SUSE*. ]]; then	
		make -C /lib/modules/`uname -r`/build M=${PWD} INSTALL_MOD_DIR=$INSTALL_MOD_DIR modules_install
	else
		ls /lib/modules/`uname -r`/extra/ &>/dev/null
		if [ $? -ne 0 ];then
			mkdir /lib/modules/`uname -r`/extra/
		fi
		cp $MODULE /lib/modules/`uname -r`/extra/
	fi

	/sbin/depmod -a
	/sbin/modprobe qlcnic

	#Install man page
	cp $QLCNIC_MAN_PAGE $QLCNIC_MAN_DIR
	chmod 644 $QLCNIC_MAN_DIR/$QLCNIC_MAN_PAGE

}

udev_install()
{
	RET=0
	diff ./${UDEV_SRC_DIR}/${UDEV_RULE_FILE} ${UDEV_RULE_DIR}/${UDEV_RULE_FILE} &> /dev/null
	RET=$?
	diff ./${UDEV_SRC_DIR}/${UDEV_SCRIPT} ${UDEV_SCRIPT_DIR}/${UDEV_SCRIPT} &> /dev/null
	(( RET += $? ))

	if [ $RET -ne 0 ]; then
		echo "** Installing udev files ${UDEV_RULE_FILE} and ${UDEV_SCRIPT} to capture FW dump **"
		cp ./${UDEV_SRC_DIR}/${UDEV_RULE_FILE} ${UDEV_RULE_DIR}/${UDEV_RULE_FILE}
		chmod 755 ${UDEV_RULE_DIR}/${UDEV_RULE_FILE}
		cp ./${UDEV_SRC_DIR}/${UDEV_SCRIPT} ${UDEV_SCRIPT_DIR}/${UDEV_SCRIPT}
		chmod 755 ${UDEV_SCRIPT_DIR}/${UDEV_SCRIPT}
		if [ -e ${UDEV_EARLY_RULE} ]; then
			cp ${UDEV_EARLY_RULE} ${UDEV_EARLY_RULE}.bak
			cat ${UDEV_EARLY_RULE} | sed "s/\(^SUBSYSTEM==\"module\".*OPTIONS=\"ignore_device\".*\)/#\1/" > ${UDEV_TMP_RULE}
			if [ -s ${UDEV_TMP_RULE} ]; then
				mv -f ${UDEV_TMP_RULE} ${UDEV_EARLY_RULE}
			fi
		fi
		${UDEV_OP}
	else
		echo "** udev rules already installed **"
	fi
}

dump_install()
{
	ls /opt | grep "qlcnic" &>/dev/null

	if [ $? -ne 0 ];then
		mkdir ${DUMP_SCRIPT_DIR}
	fi

	cp ./${UDEV_SRC_DIR}/${DUMP_SCRIPT} ${DUMP_SCRIPT_DIR}/
	chmod 755 ${DUMP_SCRIPT_DIR}/${DUMP_SCRIPT}
}

dump_uninstall()
{
	rm -f ${DUMP_SCRIPT_DIR}/${DUMP_SCRIPT} &> /dev/null
}


udev_uninstall()
{
	rm -f ${UDEV_RULE_DIR}/${UDEV_RULE_FILE} &> /dev/null
	rm -f ${UDEV_SCRIPT_DIR}/${UDEV_SCRIPT} &> /dev/null

	if [ -e ${UDEV_EARLY_RULE} ]; then
		cat ${UDEV_EARLY_RULE} | sed "s/\#\(SUBSYSTEM==\"module\".*OPTIONS=\"ignore_device\".*\)/\1/" > ${UDEV_TMP_RULE}
		if [ -s ${UDEV_TMP_RULE} ]; then
			echo "** Removing udev rule file **"
			mv -f ${UDEV_TMP_RULE} ${UDEV_EARLY_RULE}
		fi
	fi
	${UDEV_OP}
}

drv_uninstall()
{
	get_os
	drv_ver=`modinfo qlcnic | grep '^version' | awk {'print $2'}`
	rpm -qa | grep "netqlremote" &>/dev/null

	if [ $? -eq 0 ];then
		service netqlremote status | grep "running" &>/dev/null
		if [ $? -eq 0 ];then
			echo "** Stop netqlremote agent **"
			service "$CLIAGENT" stop
		fi
	fi

	modprobe -r qlcnic

	if test -f "${INSTALL_DIR}/$MODULE" ; then
		echo "** Remove module **"
		rm ${INSTALL_DIR}/$MODULE
	else
		echo "** $MODULE not found, perhaps removed in previous uninstall **"
	fi
	/sbin/depmod -a

	#Remove man page
	rm $QLCNIC_MAN_DIR/$QLCNIC_MAN_PAGE
}

print_usage()
{
	echo ""
	echo "QLogic Corporation -- Driver Install Script"
	echo ""
	echo " ./qlcnic-src-install.sh -h, --help"
	echo " Displays this usage"
	echo ""
	echo " ./qlcnic-src-install.sh -b, --build"
	echo " Compiles the driver sources and builds the driver module"
	echo ""
	echo " ./qlcnic-src-install.sh -i, --install"
	echo " Installs and loads the driver module and utilities"
	echo " -b or --build is required before -i or --install" 
	echo ""
	echo " ./qlcnic-src-install.sh -r, --remove"
	echo " Unloads and uninstalls the driver module and utilities"
}

if [ $# -ne $INPUT_ARGS ]; then
	print_usage
	exit
fi
 
case "$1" in
	-h | --help)
	print_usage
	;;
	-b | --build)
	echo "** Building qlcnic driver **"
	build_driver
	;;
	-i | --install)
	echo "** installing qlcnic driver **"
	get_os
	install_driver
	if [ $UDEVINSTALL -eq 1 ]; then
		udev_install
	fi
	dump_install
	;;
	-r | --remove)
	echo "** Removing qlcnic driver **"
	drv_uninstall
	if [ $UDEVINSTALL -eq 1 ]; then
		udev_uninstall
	fi
	dump_uninstall
	;;
	*)
	print_usage
	;;
esac
